part of pulsar_iot_client.client;

abstract class PulsarClientActorProperties {
  BrokerConnection get brokerConnection;
  String get topic;
  Int64 get actorId;

  void update(PulsarClientActorProperties newProperties);
}

abstract class PulsarClientActorCreateParams {
  String get topic;
}

abstract class PulsarClientActorCreateResult {}

abstract class PulsarClientActor<TProperties extends PulsarClientActorProperties> extends Closable {
  TProperties get properties;
  PulsarClientActorDescriptor get descriptor;

  void setConnected();
  void setDisconnected();
  void setDropped();
  void setClosed();

  void moveTo(TProperties properties);

  Exception droppedExceptionFactory();
  String get location;
}

abstract class PulsarClientActorDelegate<TActor, TProperties extends PulsarClientActorProperties, TCreateParams extends PulsarClientActorCreateParams, TCreateResult extends PulsarClientActorCreateResult> {
  TActor actorFor({
    required TProperties properties,
    required PulsarClientActorDescriptor descriptor,
    required Duration rebalanceTime,
    required ClosableActorCallback closableActorCallback
  });

  TProperties propertiesFor({
    required BrokerConnection brokerConnection,
    required Int64 id,
    required TCreateParams createParams,
    required TCreateResult createResult
  });

  Future<TCreateResult> createActorOnBroker({
    required BrokerConnection brokerConnection,
    required Int64 id,
    required TCreateParams params
  });
}

class PulsarClientActorDescriptor {
  final PulsarClientActorCreateParams createParams;
  final PulsarClientActorDelegate actorDelegate;
  final String actorName;

  PulsarClientActorDescriptor({
    required this.createParams,
    required this.actorDelegate,
    required this.actorName,
  });
}
